﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Thread2
{
    /// <summary>
    /// 定义自定义线程模型
    /// </summary>
    public abstract class BaseThread<T> where T : BaseTask
    {
        //执行线程
        Thread _Thread;
        //通知一个或多个正在等待的线程已发生事件
        ManualResetEvent mre = new ManualResetEvent(false);
        //线程安全的队列
        System.Collections.Concurrent.ConcurrentQueue<T> cqueue = new System.Collections.Concurrent.ConcurrentQueue<T>();

        /// <summary>
        /// 自定义线程ID;
        /// </summary>
        public long TID { get; set; }

        public static bool IsRuning = true;

        /// <summary>
        /// 初始化
        /// </summary>
        /// <param name="tName">线程的名称</param>
        public BaseThread(string tName)
        {
            _Thread = new Thread(Runing);
            _Thread.Name = tName;
            _Thread.Start();
        }

        //模拟新增任务
        public virtual void AddTask(T task)
        {
            //添加任务到队列
            cqueue.Enqueue(task);
            //唤醒所有相关的挂起线程
            mre.Set();
        }

        /// <summary>
        /// 子类可覆盖
        /// </summary>
        protected virtual void Runing()
        {
            //主循环 服务器运行标识
            while (IsRuning)
            {
                //如果是空则继续等待      服务器运行标识
                while (cqueue.IsEmpty && IsRuning)
                {
                    //重置线程暂停状态
                    mre.Reset();
                    //这个操作是以便服务器需要停止操作，
                    //如果停止调用线程的Thread.Abort()是会导致处理队列任务丢失
                    mre.WaitOne(2000);
#if ee
                    //为了方便测试信息打印的暂停信息
                    Console.WriteLine(DateTime.Now.ToString("HH:mm:ss:ffff") + " : " + Thread.CurrentThread.Name + " Status Sleep");
#endif
                }
                T t;
                //取出队列任务
                if (cqueue.TryDequeue(out t))
                {
                    Runing(t);
                }
            }
        }

        /// <summary>
        /// 设置运行方法为虚方法，方便子函数覆盖
        /// </summary>
        protected virtual void Runing(T run)
        {
            try
            {
                //执行任务
                run.Run();
                //打印任务信息
                Console.WriteLine(DateTime.Now.ToString("HH:mm:ss:ffff") + " : " + Thread.CurrentThread.Name + " Action：" + run.ToString());
            }
            catch (Exception ex)
            {
                //打印任务信息
                Console.WriteLine(DateTime.Now.ToString("HH:mm:ss:ffff") + " : " + Thread.CurrentThread.Name + " Action：" + run.ToString() + " Exception：" + ex);
            }
        }
    }
}
